

#    MIDAS Access Client  for the  Spectrum Service

package require MIDASAccessClient   ;#   load default support procedures

global SpectrumServerHost SpectrumServerPort
global SOAPServerHost SOAPServerPort

if {[info exists SpectrumServerHost] == 0} then {
 if {[info exists SOAPServerHost] != 0} then {
   set SpectrumServerHost $SOAPServerHost
 } else {
   set SpectrumServerHost localhost
 }
}

if {[info exists SpectrumServerPort] == 0} then {
 if {[info exists SOAPServerPort] != 0} then {
   set SpectrumServerPort $SOAPServerPort
 } else {
   set SpectrumServerPort 8015
 }
}

variable uri "urn:SpectrumService"
variable proxy "http://$SpectrumServerHost:$SpectrumServerPort/SpectrumService"

proc SpectrumClient_Configure {} {
   global SpectrumServerHost SpectrumServerPort
   variable proxy

   set proxy "http://$SpectrumServerHost:$SpectrumServerPort/SpectrumService"

   foreach method [info procs SpectrumClient__*] {catch {SOAP::configure $method -proxy $proxy}}
}

SOAP::create SpectrumClient__ServerVerbose -proxy $proxy -uri $uri -timeout 5000 \
     -name SpectrumServerVerbose \
     -params {Opt int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__ServerDebug -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecDebug \
     -params {Flag int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__ServerLogging -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecLogging \
     -params {Value int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__InquireSpectra -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecNames \
     -params {Pattern string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__CreateSpectrum -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecNew \
     -params {Name string Base int Range int Base2 int Range2 int Type int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__Create1DSpectrum -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecNew1D \
     -params {Name string Base int Range int Type int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__Create2DSpectrum -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecNew2D \
     -params {Name string Base1 int Range1 int Base2 int Range2 int Type int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__DeleteSpectrum -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecDelete \
     -params {Name string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__ZeroSpectrum -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecZero \
     -params {Name string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumDetails -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecDetails \
     -params {Name string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumGetInfo -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecGetInfo \
     -params {Name string Number int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumPutInfo -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecPutInfo \
     -params {Name string Number int Info string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumGetTitle -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecGetTitle \
     -params {Name string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumPutTitle -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecPutTitle \
     -params {Name string Title string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumGetLabel -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecGetLabel \
     -params {Name string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumPutLabel -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecPutLabel \
     -params {Name string Label string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

#      SpecGetAnnot/SpecGetAnnot - Not used by MIDAS-session

SOAP::create SpectrumClient__SpectrumGetCalib -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecGetCalib \
     -params {Name string Dim int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumPutCalib -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecPutCalib \
     -params {Name string Dim int Calib string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

#      SpecGetEffic/SpecGetEffic - Not used by MIDAS-session

SOAP::create SpectrumClient__SpectrumCompressions -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecCompressions \
     -params {Comp1 int Comp2 int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__SpectrumError -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecError \
     -params {ErrNum int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__Read1DSpectrumText -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecRead1DText \
     -params {Name string Base int Range int Size int Type int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__Read1DSpectrum -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecRead1D \
     -params {Name string Base int Range int Size int Type int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__Read2DSpectrumText -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecRead2DText \
     -params {Name string Base1 int Range1 int Base2 int Range2 int Size1 int Size2 int Type int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

SOAP::create SpectrumClient__Read2DSpectrum -proxy $proxy -uri $uri -timeout 5000 \
     -name SpecRead2D \
     -params {Name string Base1 int Range1 int Base2 int Range2 int Size1 int Size2 int Type int} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

package provide SpectrumClient 1.0
