
#    MIDAS Access Server  for  MEIS EPICS system

puts "Starting EPICS Service"

package require MIDASDomainHandler   ;#  load default support procedures
package require EPICSAccess

global MIDAS_EPICSAccessServer_Verbose
set MIDAS_EPICSAccessServer_Verbose 0

global MIDAS_EPICSChIDStore 
set MIDAS_EPICSChIDStore(dummy) ""

global EPICS_ALARM_SEVERITIES EPICS_ALARM_STATUS
set EPICS_ALARM_SEVERITIES [list NO_ALARM MINOR_ALARM MAJOR_ALARM INVALID_ALARM]
set EPICS_ALARM_STATUS [list NO_ALARM READ_ALARM WRITE_ALARM HIHI_ALARM	HIGH_ALARM LOLO_ALARM LOW_ALARM	STATE_ALARM \
   COS_ALARM COMM_ALARM	TIMEOUT_ALARM HW_LIMIT_ALARM CALC_ALARM	SCAN_ALARM LINK_ALARM SOFT_ALARM BAD_SUB_ALARM \
   UDF_ALARM DISABLE_ALARM SIMM_ALARM READ_ACCESS_ALARM WRITE_ACCESS_ALARM]

global EPICSConfigFile
set EPICSConfigFile [file join $env(MIDASBASE) MEISControls MEIS-IOC-01.db]

global EPICSStarted
set EPICSStarted 0

global env

# Use namespaces to isolate our methods
namespace eval urn:EPICSAccessServer {

   puts "Loading EPICS Access methods"

    proc AccessServerVerbose {opt} {
        do_AccessServerVerbose $opt
        return [rpcvar::rpcvar int 0]
    }

    proc Poll {} {
        do_Poll
        return [rpcvar::rpcvar int 0]
    }

    proc GetNames {} {
        set Response [do_GetNames]
        return [rpcvar::rpcvar string $Response]
    }

    proc CreateChannel {PVname} {
        set chid [do_CreateChannel $PVname]
        return [rpcvar::rpcvar int $chid]
    }

    proc ClearChannel {PVname} {
        set rc [do_ClearChannel $PVname]
        return [rpcvar::rpcvar int $rc]
    }

    proc GetInt {PVname} {
        set Response [do_GetInt $PVname]
        return [list [list Code int [first $Response]]  [list State string [second $Response]] [list Cause string [third $Response]] ]
    }

    proc PutInt {PVname vl} {
        set Response [do_PutInt $PVname $vl]
        return [list [list Code int [first $Response]]  [list State string "NO_ALARM"]]
    }

    proc GetFloat {PVname} {
        set Response [do_GetFloat $PVname]
        return [list [list Code float [first $Response]]  [list State string [second $Response]] [list Cause string [third $Response]] ]
    }

    proc PutFloat {PVname vl} {
        set Response [do_PutFloat $PVname $vl]
        return [list [list Code int [first $Response]]  [list State string "NO_ALARM"]]
    }

    proc GetString {PVname} {
        set Response [do_GetString $PVname]
        return [list [list Code string [first $Response]]  [list State string [second $Response]] [list Cause string [third $Response]] ]
    }

    proc PutString {PVname vl} {
        set Response [do_PutString $PVname $vl]
        return [list [list Code int [first $Response]]  [list State string "NO_ALARM"]]
    }

    proc do_AccessServerVerbose {option} {
        global MIDAS_EPICSAccessServer_Verbose
        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method AccessServerVerbose with parameter $option"}
        set MIDAS_EPICSAccessServer_Verbose $option
        EPICSAccess_Verbose $option
    }

    proc do_CreateChannel {PVname} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore
        global EPICSStarted

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method CreateChannel with parameter $PVname"}

        if {[info exists ChIDStore($PVname)]} {return $ChIDStore($PVname)}

        set z [catch {EPICS_CreateChannel $PVname} m]
        if {$z == 0} then {
           set ChIDStore($PVname) $m
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

        if {$EPICSStarted == 0} {return}

        for {set i 0} {$i < 1000} {incr i} {
            EPICS_Poll
            if {[EPICS_IsConnected $ChIDStore($PVname)] != 0} {break}
        }

        return $ChIDStore($PVname)

    }

    proc do_ClearChannel {PVname} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method ClearChannel with parameter $PVname"}

        if {![info exists ChIDStore($PVname)]} {return 0}

        set z [catch {EPICS_ClearChannel $ChIDStore($PVname)} m]
        if {$z == 0} then {
           unset ChIDStore($PVname)
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

        return $m
    }

    proc do_PutInt {PVname vl} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method PutInt with parameters $PVname $vl"}

        if {![info exists ChIDStore($PVname)]} {return -code error -errorcode 0x10004 -errorinfo "unknown variable"}

        set z [catch {EPICS_PutInt $ChIDStore($PVname) $vl} m]
        if {$z == 0} then {
           set Reply $m
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

       return $Reply
    }

    proc do_GetInt {PVname} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore
        global EPICS_ALARM_SEVERITIES EPICS_ALARM_STATUS

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method GetInt with parameter $PVname"}

        if {![info exists ChIDStore($PVname)]} {return -code error -errorcode 0x10004 -errorinfo "unknown variable"}

        set z [catch {EPICS_GetInt $ChIDStore($PVname)} m]
        if {$z == 0} then {
           set Reply [lindex $m 0]
           set AlarmSevr [lindex $m 1]
           set AlarmStat [lindex $m 2]
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

       return [list $Reply [lindex $EPICS_ALARM_SEVERITIES $AlarmSevr] [lindex $EPICS_ALARM_STATUS $AlarmStat]]
    }

    proc do_PutFloat {PVname vl} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method PutFloat with parameters $PVname $vl"}

        if {![info exists ChIDStore($PVname)]} {return -code error -errorcode 0x10004 -errorinfo "unknown variable"}

        set z [catch {EPICS_PutFloat $ChIDStore($PVname) $vl} m]
        if {$z == 0} then {
           set Reply $m
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

       return $Reply
    }

    proc do_GetFloat {PVname} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore
        global EPICS_ALARM_SEVERITIES EPICS_ALARM_STATUS

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method GetFloat with parameter $PVname"}

        if {![info exists ChIDStore($PVname)]} {return -code error -errorcode 0x10004 -errorinfo "unknown variable"}

        set z [catch {EPICS_GetFloat $ChIDStore($PVname)} m]
        if {$z == 0} then {
           set Reply [lindex $m 0]
           set AlarmSevr [lindex $m 1]
           set AlarmStat [lindex $m 2]
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

       return [list $Reply [lindex $EPICS_ALARM_SEVERITIES $AlarmSevr] [lindex $EPICS_ALARM_STATUS $AlarmStat]]
    }

    proc do_PutString {PVname vl} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method PutString with parameters $PVname $vl"}

        if {![info exists ChIDStore($PVname)]} {return -code error -errorcode 0x10004 -errorinfo "unknown variable"}

        set z [catch {EPICS_PutString $ChIDStore($PVname) $vl} m]
        if {$z == 0} then {
           set Reply $m
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

       return $Reply
    }

    proc do_GetString {PVname} {
        global errorCode MIDAS_EPICSAccessServer_Verbose
        upvar #0 MIDAS_EPICSChIDStore ChIDStore
        global EPICS_ALARM_SEVERITIES EPICS_ALARM_STATUS

        if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method GetString with parameter $PVname"}

        if {![info exists ChIDStore($PVname)]} {return -code error -errorcode 0x10004 -errorinfo "unknown variable"}

        set z [catch {EPICS_GetString $ChIDStore($PVname)} m]
        if {$z == 0} then {
           set Reply [lreplace $m [expr [llength $m]-2] [expr [llength $m]-1]]
           set AlarmSevr [lindex $m [expr [llength $m]-2]]
           set AlarmStat [lindex $m [expr [llength $m]-1]]
        } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n $m\n $errorCode"
        }

       return [list $Reply [lindex $EPICS_ALARM_SEVERITIES $AlarmSevr] [lindex $EPICS_ALARM_STATUS $AlarmStat]]
    }

    proc do_Poll {} {EPICS_Poll}

    proc do_GetNames {} {
        global MIDAS_EPICSAccessServer_Verbose EPICSConfigFile

       if {$MIDAS_EPICSAccessServer_Verbose} {puts "called method GetNames"}

       set Reply ""

       set z [catch {open [set EPICSConfigFile].txt r} f]
       if {$z == 0} then {
          set PVs [split [read $f] \n]
          close $f
          foreach PV $PVs {append Reply "$PV "}
       } else {
           puts "GetNames failed $f"
       }

       return $Reply

    }


    # We have to publish the public methods...

     SOAP::export AccessServerVerbose
     SOAP::export Poll
     SOAP::export CreateChannel
     SOAP::export ClearChannel
     SOAP::export PutInt
     SOAP::export GetInt
     SOAP::export PutFloat
     SOAP::export GetFloat
     SOAP::export PutString
     SOAP::export GetString

    puts "Done"

   #   create EPICS channels

    global EPICSConfigFile

      set z [catch {open [set EPICSConfigFile].txt r} f]
      if {$z == 0} then {
          set PVs [split [read $f] \n]
          close $f

          set i 0
          foreach PV $PVs {
             if {$PV == ""} {continue}
             set z [catch {do_CreateChannel $PV} m]
             if {$z == 0} then {incr i} else {puts "Channel create for $PV failed: $m"}
             EPICS_Poll
          }
          puts "$i EPICS channels created"
      } else {
          puts "Cannot open file [set EPICSConfigFile].txt\n$f"
      }

}




# register this service with tclhttpd
set z [catch {SOAP::Domain::register  -prefix  /EPICSAccessServer -namespace urn:EPICSAccessServer -uri urn:EPICSAccessServer} m]
puts "$m"

proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/EPICSAccessServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}


     proc RepeatPoll {} {catch {EPICS_Poll}; after 200 RepeatPoll}
     after 2000 RepeatPoll

set EPICSStarted 1

package provide EPICSAccessServer 1.0
